%% Solution Day 6
% Created by BH @06042012
% Edited by BB 08/10/2018 
clear
close all
 
%% Load Data

% We use readtable here because the table data structure has some nice
% benefits compared to the basic cell arrays that are returned by
% importdata.

% Use readtable. We pass the 'Delimiter','\t' argument to specify that the 
% data uses tabs as the delimiter
fileName = 'GrayBootCamp2012__mm9-(Total1,Bulent)(0hr,1hr,6hr)__(EXN,INT)-EXPRESSION.txt';
T = readtable(fileName,'Delimiter','\t');

% With a table, we don't need to worry about extracting the headers
% beforehand. We can access the data directly with the name, e.g. T.Gene or
% T{:,'Gene'}

%% Define important variables

% Define the feature that we want to analyze. We can replace this later
% with 'INT_nrds' to analyze the introns
feature = 'EXN_nrds';

%Plot option. We'll use this a lot so it's good to define at the top of
%the script
markerSize = 3; 

% Another thing we will keep doing over and over is creating a string with 
% a column name like 'EXN_nrds_Rep1_0hr'. We could do this over and over
% with square brackets, e.g.
%
%   expName = [feature '_Rep' num2str(replicate(1)) '_' ...
%       num2str(timePoint(1)) 'hr'];
%
% This will get old fast. Instead, we can define a string format and make
% it with sprintf. Check out the doc for sprintf for that %s and %d mean
expFormat = '%s_Rep%d_%dhr';
 
%% Reproducibility of replicates, Check scales
% Replicates have relative high reproducibility for highly expressing
% genes. Comparing scatter plots at linear and log scale, one might find that
% gene expression data cover a large range, so that log scale might be a
% better representation than linear scale

% Specify which data sets to compare. These could be passed in as variables
% if we were to write this script as a function
replicates = [1,2];   
times = [0,1,6]; 

figure('name', 'Reproducibility of replicates')
for iTime = 1:length(times)
    
    % Get the experiment names with sprintf
    expName1 = sprintf(expFormat,feature,replicates(1),times(iTime));
    expName2 = sprintf(expFormat,feature,replicates(2),times(iTime));    

    % First plot the raw numbers, i.e. a linear scale
    subplot(length(times),2,iTime*2-1)
    plot(T{:,expName1},T{:,expName2}, '.', 'MarkerSize', markerSize)
    xlabel(expName1,'Interpreter','none')
    ylabel(expName2,'Interpreter','none')
    
    % Add a x=y line so we can get a sense of whether there is a bias
    % between replicates
    minValue = min([get(gca,'XLim') get(gca,'YLim')]);
    maxValue = max([get(gca,'XLim') get(gca,'YLim')]);
    line([minValue maxValue],[minValue maxValue],'Color','k')
    
    % Now try a log scale with the loglog function. What does it do?
    subplot(length(times),2,iTime*2)
    loglog(T{:,expName1},T{:,expName2}, '.', 'MarkerSize', markerSize)
    xlabel(expName1,'Interpreter','none')
    ylabel(expName2,'Interpreter','none')
    
    % The ...,'Interpreter','none') was added because the experimental name
    % has underscores, e.g. EXN_nrds, which Matlab interprets with a 'TeX'
    % interpeter to print the letter 'n' as a subscript. We don't want 
    % this, so we add set the 'Interpeter' to 'none'
    
    % Add a x=y line so we can get a sense of whether there is a bias
    % between replicates
    minValue = min([get(gca,'XLim') get(gca,'YLim')]);
    maxValue = max([get(gca,'XLim') get(gca,'YLim')]);
    line([minValue maxValue],[minValue maxValue],'Color','k')
    
end

% Based on these plots, does it seem like there is an offset between
% replicates?

%% Reproducibility of relicates: MA plot
% MA plot is another way to display data. It is a type of Bland-Altman plot

% We pick a single time point to look at
iTime = 1;    % indicate time = 0hr
 
%Think of what you would do to make this work automatically for all
%timepoints

% Get the experiment names for the single time point indicated above
expName1 = sprintf(expFormat,feature,replicates(1),times(iTime));
expName2 = sprintf(expFormat,feature,replicates(2),times(iTime));    

% Calculate the mean of the log counts, so essentially the log of the 
% products, since log(ab) = log(a) + log(b) 
expMean = mean([log2(T{:,expName1}) log2(T{:,expName2})],2);

% Calculate the fold change (log difference), so essentially the log of 
% the ratio, since log(a/b) = log(a) - log(b)
expFold = log2(T{:,expName2}) - log2(T{:,expName1});

% Make plot
figure('name', 'MA Plot of replicates')
plot(expMean,expFold, '.', 'markersize', markerSize)
title(sprintf('%s_%dhr',feature,times(iTime)),'Interpreter','none')
xlabel(sprintf('Mean: Rep%d vs. Rep%d',replicates)) 
ylabel(sprintf('Fold change: Rep%d vs. Rep%d',replicates)) 

%Add a horizontal line, which would be no change between replicate
line([0 20],[0 0],'Color','k')

%Do you see an offset, i.e. is the distribution not centered on the
%horizontal line?
 
%% Removing zero  expression reads

% To simplify the analysis, genes with zero expression reads are removed.
% We'll make a copy of the Table first and only concern ourselves with the 
% 0 and 6 hr time points since these will be the ones we'll use later
times = [0 6];
replicates = [1,2];


expNames = {};
isZero = false(size(T,1),1);
for iTime = 1:length(times)
    
    %Experiment names for the two replicates
    expNames{2*iTime-1} = sprintf(expFormat,feature,replicates(1),times(iTime));
    expNames{2*iTime} = sprintf(expFormat,feature,replicates(2),times(iTime));
    
    %Check which rows have zeros
    isZero = isZero | T{:,expNames{2*iTime-1}}==0;
    isZero = isZero | T{:,expNames{2*iTime}}==0;
    
end

% Use expNames and isZero to get the table copy. We keep the info columns
% (1-12) and then only the data from the experiments of interest. 
Tcopy = [T(~isZero,1:12) T(~isZero,expNames)];


%% Perform normalization between replicates

% Because there is an offset, we can normalize.

% Define a smoothing span. This number is a percentage of the data to 
% consider when smoothing
span = 0.25; 

% The exist function will see if there is a function named "smooth" on
% the path. We use ~exist to see if it doesn't exist
if ~exist('smooth')
    error('Must have Curve Fitting toolbox for smooth function')
end

% Do normalization for each time period
for iTime = 1:length(times)
     
    % Calculate the fold change (log difference), so essentially the log of 
    % the ratio, since log(a/b) = log(a) - log(b)
    dataRep1 = log2(Tcopy{:,expNames{2*iTime-1}});
    dataRep2 = log2(Tcopy{:,expNames{2*iTime}});
    
    expFold = dataRep2 - dataRep1;
    expMean = mean([dataRep1 dataRep2],2);
    
    % We will smooth this noisy data to get an overall distribution bias
    % Smoothing is a complicated topic that deserves it's own series of
    % lectures, so don't let our smoothing method below make you think this 
    % is the only way or even the right way to smooth this data
    
    %Smooth the expFold data    
    smoothedDiff = smooth(expMean,expFold,span,'lowess'); 
    
    %Before normalization plot
    figure('name','Normalization');
    subplot(1,2,1)
    plot(expMean,expFold,'.','MarkerSize',markerSize);
    title(sprintf('Before normalization: %s, %dhr',feature,times(iTime)), ...
        'Interpreter','none')
    xlabel(sprintf('Mean: Rep%d vs. Rep%d',replicates)) 
    ylabel(sprintf('Fold change: Rep%d vs. Rep%d',replicates)) 
    
    %Add the smoothed data with red dots and a line at y = 0
    hold on; 
    line([0 18],[0 0],'Color','k')
    plot(expMean,smoothedDiff,'r.','LineWidth',2)    

    % Nudge Rep2 down halfway and Rep1 up halfway. Then repeat fold calculation
    dataRep2 = dataRep2 - smoothedDiff/2;
    dataRep1 = dataRep1 + smoothedDiff/2;
    expFold = dataRep2 - dataRep1;    
    
    % One more call to smooth to make sure our normalization worked. 
    % The red dots should now lie on top of the horizontal black line
    smoothedDiff = smooth(expMean,expFold,span,'lowess');

    %Redo plot to show the difference
    subplot(1,2,2)    
    plot(expMean,expFold,'.','MarkerSize',markerSize);
    title(sprintf('After normalization: %s_%dhr',feature,times(iTime)), ...
        'Interpreter','none')
    xlabel(sprintf('Mean: Rep%d vs. Rep%d',replicates)) 
    ylabel(sprintf('Fold change: Rep%d vs. Rep%d',replicates)) 

    %Add the new smoothed data with red dots and a line at y = 0
    hold on; 
    line([0 18],[0 0],'Color','k')
    plot(expMean,smoothedDiff,'r.','LineWidth',2)            

    % Add the normalized data to Tcopy so we can use it later. We rename
    % the experiment names so we can resuse the expName variable
    expNames{2*iTime-1} = [expNames{2*iTime-1} '_lognorm'];
    Tcopy{:,expNames{2*iTime-1}} = dataRep1;
    
    expNames{2*iTime} = [expNames{2*iTime} '_lognorm'];
    Tcopy{:,expNames{2*iTime}} = dataRep2;

end
 
%% Display EXN nrds 0hr vs 6hr: MA plot
% MA plot. One might find that for highly expressed genes, expression reads
% are slightly lower than ones at 0 hour

% Take the average of the two replicates 
dataTime1 = mean([Tcopy{:,expNames{1}} Tcopy{:,expNames{2}}],2);
dataTime2 = mean([Tcopy{:,expNames{3}} Tcopy{:,expNames{4}}],2);

% Calculate M and A
M = dataTime2 - dataTime1;
A = mean([dataTime1 dataTime2],2);

% Compare the two times
figure('name','MA plot of time points')
plot(A,M,'.','MarkerSize',markerSize)
xlabel('Mean')
ylabel('Fold change')
title(sprintf('%s:  %dhr vs. %dhr',feature,times),'Interpreter','none')
line([0 18],[0 0],'Color','k')
 
% We can find the genes with the largest changes by finding the values
% farthest away from the horizontal line. The maxk function will find the
% largest k values of an array. 
nOutlier = 5;
[~,outliers] = maxk(abs(M),nOutlier);
outlierNames = Tcopy{outliers,'Gene'};
text(A(outliers),M(outliers),outlierNames);

%% Display EXN nrds 0hr vs 6hr
% We could also do a scatter plot like before

% Make scatter plot
figure('name','Scatter plot of time points')
plot(dataTime1,dataTime2,'.','MarkerSize',markerSize)
xlabel(sprintf('%dhr, log2 scale',times(1)))
ylabel(sprintf('%dhr, log2 scale',times(2)))
title(sprintf('%s:  %dhr vs. %dhr',feature,times),'Interpreter','none')

%Add the x=y line
minValue = min([get(gca,'XLim') get(gca,'YLim')]);
maxValue = max([get(gca,'XLim') get(gca,'YLim')]);
line([minValue maxValue],[minValue maxValue],'Color','k')

% Label the same outliers as before. These are the points farthest away 
% from the x=y line
text(dataTime1(outliers),dataTime2(outliers),outlierNames);


%% Determine significance of fold change: Permutation test
% We can do a permutation test to see the significance of the fold change.

% This is the actual fold change that we will be testing
actualChange = dataTime2 - dataTime1;

% For each gene, we have 4 data points (2 replicates, 2 time points) to
% calcualte a fold change. If we number these 1,2,3,4 then there are 6 
% possible permutations to calculate the fold change: 
% 
%   mean(1,2) - mean(3,4)
%   mean(1,3) - mean(2,4)
%   mean(1,4) - mean(2,3)
%   mean(2,3) - mean(1,4)
%   mean(2,4) - mean(1,3)
%   mean(3,4) - mean(1,2)
%
% This is essentially (4 choose 2), so we can choose the nchoosek function
% to get out permutations


% Each row in this array is two of the numbers from 1:4. They will be the 
% first pair. We can get the second pair by finding the other numbers 
pairs = nchoosek(1:4,2);

nGenes = length(dataTime1);
shuffledChange = zeros(nGenes,size(pairs,1));
for iPair = 1:size(pairs)
    
    firstPair = pairs(iPair,:);
    secondPair = find(~ismember(1:4,firstPair));
    mean1 = mean([Tcopy{:,expNames{firstPair(1)}} ...
                  Tcopy{:,expNames{firstPair(2)}}],2);
    mean2 = mean([Tcopy{:,expNames{secondPair(1)}} ...
                  Tcopy{:,expNames{secondPair(2)}}],2);
    shuffledChange(:,iPair) = mean1 - mean2;
  
end

% Now we iterate through the genes to find how many of these shuffled fold
% change values are as extreme or greater than the actual fold change. This
% makes a one sided p-value. How could we make this 2-sided?
pValues = zeros(nGenes,1);
for iGene = 1:nGenes
    pValues(iGene) = sum(shuffledChange(:) >= actualChange(iGene)) ...
        ./ numel(shuffledChange);
end

%% Find significant genes

% We can plot the pvalues to visualize how many are below some threshold
% alpha
alpha = 0.001; 
[sortpValues,sortInd] = sort(pValues,'ascend');


figure('name','p-values')
semilogy(sortpValues, 'o')
line([1 nGenes],[alpha alpha],'Color','red')
ylabel('Fold change p-value')
xlabel('Index')

%Which genes are significant 
sigChanges = sortpValues < alpha;
sigGenes = Tcopy{:,'Gene'}(sortInd(sigChanges))
nSigGenes = sum(sigChanges)
